#ifndef __QTCURVE_H__
#define __QTCURVE_H__

/*
  QtCurve (C) Craig Drummond, 2003   Craig.Drummond@lycos.co.uk

  ----

  Based upon B???Curve style (Copyright (c) 2002 R?d H?t, Inc)
      Bernhard Rosenkrnzer <bero@r?dh?t.com>
      Preston Brown <pbrown@r?dh?t.com>
      Than Ngo <than@r?dh?t.com>

  Released under the GNU General Public License (GPL) v2.

  ----

  B???Curve is based on the KDE Light style, 2nd revision:
  Copyright (c) 2000-2001 Trolltech AS (info@trolltech.com)

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*/

#include <kstyle.h>
#include <qcolor.h>

class QtCurveStyle : public KStyle
{
	Q_OBJECT

public:
    enum
    {
        GRADIENT_BASE,
        GRADIENT_LIGHT,
        GRADIENT_DARK,
        GRADIENT_TOP,
        GRADIENT_BOTTOM,
        GRADIENT_NUM_COLS
    };

	QtCurveStyle(bool bb=false, bool bf=false);
	virtual ~QtCurveStyle() {}

        void polish(QPalette &pal);
	void polish(QWidget *widget);
	void unPolish(QWidget *widget);

	void drawLightBevel(QPainter *p, const QRect &r,
                           QStyle::SFlags flags, const QColor *fill = NULL, const QColor *custom = NULL) const;

	void drawPrimitive(PrimitiveElement, QPainter *, const QRect &,
	                   const QColorGroup &, SFlags = Style_Default,
	                   const QStyleOption & = QStyleOption::Default) const;

        void drawKStylePrimitive( KStylePrimitive kpe,
                                          QPainter* p,
                                          const QWidget* widget,
                                          const QRect &r,
                                          const QColorGroup &cg,
                                          SFlags flags,
                                          const QStyleOption &opt ) const;

	void drawControl(ControlElement, QPainter *, const QWidget *,
	                 const QRect &, const QColorGroup &,
	                 SFlags = Style_Default,
	                 const QStyleOption & = QStyleOption::Default) const;

	void drawControlMask(ControlElement, QPainter *, const QWidget *,
	                     const QRect &,
	                     const QStyleOption & = QStyleOption::Default) const;

	QRect subRect(SubRect, const QWidget *) const;

	void drawComplexControl(ComplexControl, QPainter *, const QWidget *,
	                        const QRect &, const QColorGroup &,
	                        SFlags = Style_Default,
	                        SCFlags = SC_All, SCFlags = SC_None,
	                        const QStyleOption & = QStyleOption::Default) const;

	QRect querySubControlMetrics(ComplexControl, const QWidget *,
	                             SubControl, const QStyleOption & = QStyleOption::Default) const;

	SubControl querySubControl(ComplexControl, const QWidget *, const QPoint &,
	                           const QStyleOption &data = QStyleOption::Default) const;

	int pixelMetric(PixelMetric, const QWidget * = 0) const;
        int kPixelMetric(KStylePixelMetric kpm, const QWidget *widget) const;

	QSize sizeFromContents(ContentsType, const QWidget *, const QSize &,
	                       const QStyleOption & = QStyleOption::Default) const;

	int styleHint(StyleHint, const QWidget * = 0,
	              const QStyleOption & = QStyleOption::Default,
	              QStyleHintReturn * = 0 ) const;

protected:
	bool eventFilter(QObject *object, QEvent *event);
	void drawGradient(QPainter *p, QRect const &r, bool horiz=true) const;
        void drawSliderHandle(QPainter *p, const QRect &r, QStyle::SFlags flags) const;
        void drawSliderGroove(QPainter *p, const QRect &r, QStyle::SFlags flags, const QWidget *widget) const;

private:
    QColor  gradient[GRADIENT_NUM_COLS],
            gray[7], button[7];   // Last color = base color, for comparisons!
    bool    isKicker,
            borderButton,
            borderFrame;
    QWidget *hoverWidget;
};

#endif
